syntax = "proto3";

package envoy.extensions.filters.common.matcher.action.v3;

import "udpa/annotations/status.proto";

option java_package = "io.envoyproxy.envoy.extensions.filters.common.matcher.action.v3";
option java_outer_classname = "SkipActionProto";
option java_multiple_files = true;
option go_package = "github.com/envoyproxy/go-control-plane/envoy/extensions/filters/common/matcher/action/v3;actionv3";
option (udpa.annotations.file_status).package_version_status = ACTIVE;

// [#protodoc-title: Common Match Actions]

// Configuration for the SkipFilter match action. When matching results in this action, the
// associated filter will be ignored for all filter callbacks (e.g. `encodeHeaders`, `encodeData`,
// etc. for HTTP filters) after the matcher arrives at the match, including the callback that
// caused the match result. For example, when used with a HTTP filter and the match result was
// resolved after receiving the HTTP response headers, the HTTP filter will *not* receive the
// response header callback.
//
// As a result, if this match action is resolved before the first filter callback (e.g. HTTP request
// headers), the filter will be completely skipped.
message SkipFilter {
}
