syntax = "proto3";

package envoy.type.v3;

import "udpa/annotations/status.proto";

option java_package = "io.envoyproxy.envoy.type.v3";
option java_outer_classname = "HttpProto";
option java_multiple_files = true;
option go_package = "github.com/envoyproxy/go-control-plane/envoy/type/v3;typev3";
option (udpa.annotations.file_status).package_version_status = ACTIVE;

// [#protodoc-title: HTTP]

enum CodecClientType {
  HTTP1 = 0;

  HTTP2 = 1;

  // [#not-implemented-hide:] QUIC implementation is not production ready yet. Use this enum with
  // caution to prevent accidental execution of QUIC code. I.e. `!= HTTP2` is no longer sufficient
  // to distinguish HTTP1 and HTTP2 traffic.
  HTTP3 = 2;
}
