syntax = "proto3";

package envoy.extensions.filters.http.alternate_protocols_cache.v3;

import "envoy/config/core/v3/protocol.proto";

import "udpa/annotations/status.proto";

option java_package = "io.envoyproxy.envoy.extensions.filters.http.alternate_protocols_cache.v3";
option java_outer_classname = "AlternateProtocolsCacheProto";
option java_multiple_files = true;
option go_package = "github.com/envoyproxy/go-control-plane/envoy/extensions/filters/http/alternate_protocols_cache/v3;alternate_protocols_cachev3";
option (udpa.annotations.file_status).package_version_status = ACTIVE;

// [#protodoc-title: Alternate Protocols Cache]

// Configuration for the alternate protocols cache HTTP filter.
// [#extension: envoy.filters.http.alternate_protocols_cache]
message FilterConfig {
  // If set, causes the use of the alternate protocols cache, which is responsible for
  // parsing and caching HTTP Alt-Svc headers. This enables the use of HTTP/3 for upstream
  // servers that advertise supporting it.
  // TODO(RyanTheOptimist): Make this field required when HTTP/3 is enabled via auto_http.
  config.core.v3.AlternateProtocolsCacheOptions alternate_protocols_cache_options = 1;
}
