syntax = "proto3";

package envoy.extensions.load_balancing_policies.wrr_locality.v3;

import "envoy/config/cluster/v3/cluster.proto";

import "udpa/annotations/status.proto";
import "validate/validate.proto";

option java_package = "io.envoyproxy.envoy.extensions.load_balancing_policies.wrr_locality.v3";
option java_outer_classname = "WrrLocalityProto";
option java_multiple_files = true;
option go_package = "github.com/envoyproxy/go-control-plane/envoy/extensions/load_balancing_policies/wrr_locality/v3;wrr_localityv3";
option (udpa.annotations.file_status).package_version_status = ACTIVE;

// [#protodoc-title: Weighted Round Robin Locality-Picking Load Balancing Policy]

// Configuration for the wrr_locality LB policy. See the :ref:`load balancing architecture overview
// <arch_overview_load_balancing_types>` for more information.
// [#extension: envoy.clusters.lb_policy]
message WrrLocality {
  // The child LB policy to create for endpoint-picking within the chosen locality.
  config.cluster.v3.LoadBalancingPolicy endpoint_picking_policy = 1
      [(validate.rules).message = {required: true}];
}
