syntax = "proto3";

package envoy.api.v2.core;

import "udpa/annotations/migrate.proto";
import "udpa/annotations/status.proto";
import "validate/validate.proto";

option java_package = "io.envoyproxy.envoy.api.v2.core";
option java_outer_classname = "GrpcMethodListProto";
option java_multiple_files = true;
option go_package = "github.com/envoyproxy/go-control-plane/envoy/api/v2/core";
option (udpa.annotations.file_migrate).move_to_package = "envoy.config.core.v3";
option (udpa.annotations.file_status).package_version_status = FROZEN;

// [#protodoc-title: gRPC method list]

// A list of gRPC methods which can be used as an allowlist, for example.
message GrpcMethodList {
  message Service {
    // The name of the gRPC service.
    string name = 1 [(validate.rules).string = {min_bytes: 1}];

    // The names of the gRPC methods in this service.
    repeated string method_names = 2 [(validate.rules).repeated = {min_items: 1}];
  }

  repeated Service services = 1;
}
