syntax = "proto3";

package envoy.extensions.stat_sinks.wasm.v3;

import "envoy/extensions/wasm/v3/wasm.proto";

import "udpa/annotations/status.proto";

option java_package = "io.envoyproxy.envoy.extensions.stat_sinks.wasm.v3";
option java_outer_classname = "WasmProto";
option java_multiple_files = true;
option go_package = "github.com/envoyproxy/go-control-plane/envoy/extensions/stat_sinks/wasm/v3;wasmv3";
option (udpa.annotations.file_status).package_version_status = ACTIVE;

// [#protodoc-title: Wasm]
// Wasm :ref:`configuration overview <config_stat_sinks_wasm>`.
// [#extension: envoy.stat_sinks.wasm]

message Wasm {
  // General Plugin configuration.
  envoy.extensions.wasm.v3.PluginConfig config = 1;
}
