syntax = "proto3";

package envoy.config.filter.http.grpc_web.v2;

import "udpa/annotations/migrate.proto";
import "udpa/annotations/status.proto";

option java_package = "io.envoyproxy.envoy.config.filter.http.grpc_web.v2";
option java_outer_classname = "GrpcWebProto";
option java_multiple_files = true;
option go_package = "github.com/envoyproxy/go-control-plane/envoy/config/filter/http/grpc_web/v2;grpc_webv2";
option (udpa.annotations.file_migrate).move_to_package =
    "envoy.extensions.filters.http.grpc_web.v3";
option (udpa.annotations.file_status).package_version_status = FROZEN;

// [#protodoc-title: gRPC Web]
// gRPC Web :ref:`configuration overview <config_http_filters_grpc_web>`.
// [#extension: envoy.filters.http.grpc_web]

// gRPC Web filter config.
message GrpcWeb {
}
