syntax = "proto3";

package envoy.config.filter.http.dynamo.v2;

import "udpa/annotations/migrate.proto";
import "udpa/annotations/status.proto";

option java_package = "io.envoyproxy.envoy.config.filter.http.dynamo.v2";
option java_outer_classname = "DynamoProto";
option java_multiple_files = true;
option go_package = "github.com/envoyproxy/go-control-plane/envoy/config/filter/http/dynamo/v2;dynamov2";
option (udpa.annotations.file_migrate).move_to_package = "envoy.extensions.filters.http.dynamo.v3";
option (udpa.annotations.file_status).package_version_status = FROZEN;

// [#protodoc-title: Dynamo]
// Dynamo :ref:`configuration overview <config_http_filters_dynamo>`.
// [#extension: envoy.filters.http.dynamo]

// Dynamo filter config.
message Dynamo {
}
