syntax = "proto3";

package envoy.annotations;
option go_package = "github.com/envoyproxy/go-control-plane/envoy/annotations";

import "google/protobuf/descriptor.proto";

// [#protodoc-title: Resource]

// Magic number in this file derived from top 28bit of SHA256 digest of "envoy.annotation.resource".
extend google.protobuf.ServiceOptions {
  ResourceAnnotation resource = 265073217;
}

message ResourceAnnotation {
  // Annotation for xDS services that indicates the fully-qualified Protobuf type for the resource
  // type.
  string type = 1;
}
