syntax = "proto3";

package envoy.type;

import "udpa/annotations/status.proto";
import "validate/validate.proto";

option java_package = "io.envoyproxy.envoy.type";
option java_outer_classname = "HashPolicyProto";
option java_multiple_files = true;
option go_package = "github.com/envoyproxy/go-control-plane/envoy/type";
option (udpa.annotations.file_status).package_version_status = FROZEN;

// [#protodoc-title: Hash Policy]

// Specifies the hash policy
message HashPolicy {
  // The source IP will be used to compute the hash used by hash-based load balancing
  // algorithms.
  message SourceIp {
  }

  oneof policy_specifier {
    option (validate.required) = true;

    SourceIp source_ip = 1;
  }
}
