syntax = "proto3";

package envoy.extensions.config.validators.minimum_clusters.v3;

import "udpa/annotations/status.proto";

option java_package = "io.envoyproxy.envoy.extensions.config.validators.minimum_clusters.v3";
option java_outer_classname = "MinimumClustersProto";
option java_multiple_files = true;
option go_package = "github.com/envoyproxy/go-control-plane/envoy/extensions/config/validators/minimum_clusters/v3;minimum_clustersv3";
option (udpa.annotations.file_status).package_version_status = ACTIVE;

// [#protodoc-title: Minimum Clusters]
// [#extension: envoy.config.validators.minimum_clusters]

// Validates a CDS config, and ensures that the number of clusters is above the
// set threshold.
message MinimumClustersValidator {
  // The minimal clusters threshold. Any CDS config update leading to less than
  // this number will be rejected.
  // Default value is 0.
  uint32 min_clusters_num = 1;
}
