syntax = "proto3";

package envoy.extensions.retry.host.previous_hosts.v3;

import "udpa/annotations/status.proto";
import "udpa/annotations/versioning.proto";

option java_package = "io.envoyproxy.envoy.extensions.retry.host.previous_hosts.v3";
option java_outer_classname = "PreviousHostsProto";
option java_multiple_files = true;
option go_package = "github.com/envoyproxy/go-control-plane/envoy/extensions/retry/host/previous_hosts/v3;previous_hostsv3";
option (udpa.annotations.file_status).package_version_status = ACTIVE;

// [#protodoc-title: Previous Hosts Predicate]
// [#extension: envoy.retry_host_predicates.previous_hosts]

message PreviousHostsPredicate {
  option (udpa.annotations.versioning).previous_message_type =
      "envoy.config.retry.previous_hosts.v2.PreviousHostsPredicate";
}
