syntax = "proto3";

package envoy.api.v2.listener;

import public "envoy/api/v2/listener/listener_components.proto";

option java_package = "io.envoyproxy.envoy.api.v2.listener";
option java_outer_classname = "ListenerProto";
option java_multiple_files = true;
option go_package = "github.com/envoyproxy/go-control-plane/envoy/api/v2/listener";
option csharp_namespace = "Envoy.Api.V2.ListenerNS";
option ruby_package = "Envoy::Api::V2::ListenerNS";
