syntax = "proto3";

package envoy.config.filter.network.sni_cluster.v2;

import "udpa/annotations/migrate.proto";
import "udpa/annotations/status.proto";

option java_package = "io.envoyproxy.envoy.config.filter.network.sni_cluster.v2";
option java_outer_classname = "SniClusterProto";
option java_multiple_files = true;
option go_package = "github.com/envoyproxy/go-control-plane/envoy/config/filter/network/sni_cluster/v2;sni_clusterv2";
option (udpa.annotations.file_migrate).move_to_package =
    "envoy.extensions.filters.network.sni_cluster.v3";
option (udpa.annotations.file_status).package_version_status = FROZEN;

// [#protodoc-title: SNI Cluster Filter]
// Set the upstream cluster name from the SNI field in the TLS connection.
// [#extension: envoy.filters.network.sni_cluster]

message SniCluster {
}
