syntax = "proto3";

package envoy.extensions.filters.network.rocketmq_proxy.v3;

import "contrib/envoy/extensions/filters/network/rocketmq_proxy/v3/route.proto";

import "google/protobuf/duration.proto";

import "udpa/annotations/status.proto";
import "validate/validate.proto";

option java_package = "io.envoyproxy.envoy.extensions.filters.network.rocketmq_proxy.v3";
option java_outer_classname = "RocketmqProxyProto";
option java_multiple_files = true;
option go_package = "github.com/envoyproxy/go-control-plane/envoy/extensions/filters/network/rocketmq_proxy/v3;rocketmq_proxyv3";
option (udpa.annotations.file_status).package_version_status = ACTIVE;

// [#protodoc-title: RocketMQ Proxy]
// RocketMQ Proxy :ref:`configuration overview <config_network_filters_rocketmq_proxy>`.
// [#extension: envoy.filters.network.rocketmq_proxy]

message RocketmqProxy {
  // The human readable prefix to use when emitting statistics.
  string stat_prefix = 1 [(validate.rules).string = {min_len: 1}];

  // The route table for the connection manager is specified in this property.
  RouteConfiguration route_config = 2;

  // The largest duration transient object expected to live, more than 10s is recommended.
  google.protobuf.Duration transient_object_life_span = 3;

  // If develop_mode is enabled, this proxy plugin may work without dedicated traffic intercepting
  // facility without considering backward compatibility of exiting RocketMQ client SDK.
  bool develop_mode = 4;
}
